AGENT_ACTIVE_MODE = 'STOOBLY_AGENT_ACTIVE_MODE'
AGENT_CONFIG_PATH = 'STOOBLY_AGENT_CONFIG_PATH'
AGENT_ENABLED = 'STOOBLY_AGENT_ENABLED'
AGENT_SELF_INTERCEPT_ENABLED = 'STOOBLY_AGENT_SELF_INTERCEPT_ENABLED'
AGENT_INCLUDE_PATTERNS = 'STOOBLY_AGENT_INCLUDE_PATTERNS'
AGENT_IS_HEADLESS = 'STOOBLY_AGENT_IS_HEADLESS'
AGENT_EXCLUDE_PATTERNS = 'STOOBLY_AGENT_EXCLUDE_PATTERNS'
AGENT_LIFECYCLE_HOOKS_PATH = 'STOOBLY_AGENT_LIFECYCLE_HOOKS_PATH'
AGENT_POLICY = 'STOOBLY_AGENT_POLICY'
AGENT_PROJECT_KEY = 'STOOBLY_AGENT_PROJECT_KEY'
AGENT_PROXY_URL = 'STOOBLY_AGENT_PROXY_URL'
AGENT_PUBLIC_DIRECTORY_PATH = 'STOOBLY_AGENT_PUBLIC_DIRECTORY_PATH'
AGENT_REMOTE_ENABLED = 'STOOBLY_AGENT_REMOTE_ENABLED'
AGENT_REMOTE_PROJECT_KEY = 'STOOBLY_AGENT_REMOTE_PROJECT_KEY'
AGENT_REPLAY_HOST = 'STOOBLY_AGENT_REPLAY_HOST'
AGENT_REPLAY_SCHEME = 'STOOBLY_AGENT_REPLAY_SCHEME'
AGENT_RESPONSE_FIXTURES_PATH = 'STOOBLY_AGENT_RESPONSE_FIXTURES_PATH'
AGENT_SERVICE_URL = 'STOOBLY_AGENT_SERVICE_URL'
AGENT_SCENARIO_KEY = 'STOOBLY_AGENT_SCENARIO_KEY'
AGENT_SIMULATE_LATENCY = 'STOOBLY_AGENT_SIMULATE_LATENCY'
AGENT_URL = 'STOOBLY_AGENT_URL'
API_URL = 'STOOBLY_API_URL'
API_KEY = 'STOOBLY_API_KEY'
ENV = 'STOOBLY_AGENT_ENV'
FEATURE_REMOTE = 'STOOBLY_FETURE_REMOTE'
HTTP_PROXY = 'HTTP_PROXY'
HTTPS_PROXY = 'HTTPS_PROXY'
LOG_LEVEL = 'STOOBLY_AGENT_LOG_LEVEL'
NO_PROXY = 'NO_PROXY'