ALIAS_RESOLVE_STRATEGY = 'X-Stoobly-Alias-Resolve-Strategy'
CONTENT_TYPE = 'X-Stoobly-Content-Type'
CONTENT_TYPE_TEST_RESULTS = 'test/results'
REMOTE_PROJECT_KEY = 'X-Stoobly-Endpoints-Project-Id'
MOCK_POLICY = 'X-Stoobly-Mock-Policy'
MOCK_REQUEST_ID = 'X-Stoobly-Request-Id'
MOCK_REQUEST_ENDPOINT_ID = 'X-Stoobly-Request-Endpoint-Id'
MOCK_REQUEST_KEY = 'X-Stoobly-Request-Key'
DO_PROXY = 'X-Stoobly-Do-Proxy'
LIFECYCLE_HOOKS_PATH = 'X-Stoobly-Lifecycle-Hooks-Path'
PROJECT_KEY = 'X-Stoobly-Project-Key'
PROXY_MODE = 'X-Stoobly-Proxy-Mode'
PUBLIC_DIRECTORY_PATH = 'X-Stoobly-Public-Directory-Path'
RECORD_POLICY = 'X-Stoobly-Record-Policy'
REPORT_KEY = 'X-Stoobly-Report-Key'
REQUEST_ORIGIN = 'X-Stoobly-Request-Origin'
RESPONSE_FIXTURES_PATH = 'X-Stoobly-Response-Fixtures-Path'
RESPONSE_ID = 'X-Stoobly-Response-Id'
RESPONSE_LATENCY = 'X-Stoobly-Request-Response-Latency'
RESPONSE_PROXY_MODE = 'X-Stoobly-Response-Proxy-Mode'
SCENARIO_KEY = 'X-Stoobly-Scenario-Key'
SESSION_ID = 'X-Stoobly-Session-Id'
SERVICE_URL = 'X-Stoobly-Service-Url'
TEST_FILTER = 'X-Stoobly-Test-Filter'
TEST_ID = 'X-Stoobly-Test-Id'
TEST_SAVE_RESULTS = 'X-Stoobly-Test-Save-Results'
TEST_SKIP = 'X-Stoobly-Test-Skip'
TEST_STRATEGY = 'X-Stoobly-Test-Strategy'
TRACE_ID = 'X-Stoobly-Trace-Id'
TRACE_REQUEST_ID = 'X-Stoobly-Trace-Request-Id'