!function(){function n(n,t){for(var u=0;u<t.length;u++){var e=t[u];e.enumerable=e.enumerable||!1,e.configurable=!0,"value"in e&&(e.writable=!0),Object.defineProperty(n,e.key,e)}}function t(n,t){if(!(n instanceof t))throw new TypeError("Cannot call a class as a function")}(window.webpackJsonp=window.webpackJsonp||[]).push([[45],{J5l0:function(u,e,l){"use strict";l.r(e),l.d(e,"OauthCallbackModuleNgFactory",function(){return k});var o=l("8Y7J"),c=function n(){t(this,n)},a=l("pMnS"),r=function(){function u(n,e){t(this,u),this.router=n,this.tokenService=e}var e,l,o;return e=u,(l=[{key:"ngOnInit",value:function(){this.tokenService.processOAuthCallback(),this.router.navigate(["/"])}}])&&n(e.prototype,l),o&&n(e,o),u}(),i=l("iInd"),b=l("hU4o"),s=o.yb({encapsulation:0,styles:[[""]],data:{}});function f(n){return o.bc(0,[],null,null)}var p=o.wb("oauth-callback",r,function(n){return o.bc(0,[(n()(),o.Ab(0,0,null,null,1,"oauth-callback",[],null,null,null,f,s)),o.zb(1,114688,null,0,r,[i.p,b.c],null,null)],function(n,t){n(t,1,0)},null)},{},{},[]),h=l("SVse"),w=function n(){t(this,n)},k=o.xb(c,[],function(n){return o.Lb([o.Mb(512,o.j,o.bb,[[8,[a.a,p]],[3,o.j],o.z]),o.Mb(4608,h.q,h.p,[o.w]),o.Mb(1073742336,h.c,h.c,[]),o.Mb(1073742336,i.t,i.t,[[2,i.z],[2,i.p]]),o.Mb(1073742336,w,w,[]),o.Mb(1073742336,c,c,[]),o.Mb(1024,i.n,function(){return[[{path:"",component:r}]]},[])])})}}])}();