(window.webpackJsonp=window.webpackJsonp||[]).push([[45],{J5l0:function(n,t,l){"use strict";l.r(t),l.d(t,"OauthCallbackModuleNgFactory",function(){return k});var u=l("8Y7J");class c{}var a=l("pMnS");class r{constructor(n,t){this.router=n,this.tokenService=t}ngOnInit(){this.tokenService.processOAuthCallback(),this.router.navigate(["/"])}}var o=l("iInd"),b=l("hU4o"),e=u.yb({encapsulation:0,styles:[[""]],data:{}});function s(n){return u.bc(0,[],null,null)}function i(n){return u.bc(0,[(n()(),u.Ab(0,0,null,null,1,"oauth-callback",[],null,null,null,s,e)),u.zb(1,114688,null,0,r,[o.p,b.c],null,null)],function(n,t){n(t,1,0)},null)}var p=u.wb("oauth-callback",r,i,{},{},[]),h=l("SVse");class M{}var k=u.xb(c,[],function(n){return u.Lb([u.Mb(512,u.j,u.bb,[[8,[a.a,p]],[3,u.j],u.z]),u.Mb(4608,h.q,h.p,[u.w]),u.Mb(1073742336,h.c,h.c,[]),u.Mb(1073742336,o.t,o.t,[[2,o.z],[2,o.p]]),u.Mb(1073742336,M,M,[]),u.Mb(1073742336,c,c,[]),u.Mb(1024,o.n,function(){return[[{path:"",component:r}]]},[])])})}}]);