request_show_response = {
  "created_at": "2022-06-08T04:06:58.000Z",
  "endpoint_id": 1308,
  "host": "alpha.api.stoobly.com",
  "http_version": 1.1,
  "id": 11131,
  "latency": 373,
  "method": "GET",
  "path": "/requests/11109/responses/mock",
  "port": "443",
  "position": 7,
  "project_id": 37,
  "query": "project_id=38",
  "scenario_id": 289,
  "scheme": "https",
  "starred": False,
  "status": 200,
  "updated_at": "2022-06-08T04:18:04.995Z",
  "url": "https://alpha.api.stoobly.com/requests/11109/responses/mock?project_id=38",
  "components": [
      "query_params",
      "response"
  ],
  "endpoint": "/requests/:requestId/responses/mock",
  "key": "eyJpIjoxMTEzMSwicCI6MzcsInQiOjU2MTc0MDN9"
}