AFTER_MOCK = 'handle_after_mock'
AFTER_RECORD = 'handle_after_record'
AFTER_REPLAY = 'handle_after_replay'
AFTER_TEST = 'handle_after_test'
BEFORE_MOCK = 'handle_before_mock'
BEFORE_RECORD = 'handle_before_record'
BEFORE_REPLAY = 'handle_before_replay'
BEFORE_REQUEST = 'handle_before_request'
BEFORE_RESPONSE = 'handle_before_response'
BEFORE_TEST = 'handle_before_test'
ON_LENGTH_MATCH_ERROR = 'handle_length_match_error'
ON_PARAM_NAME_EXISTS_ERROR = 'handle_param_name_exists_error'
ON_PARAM_NAME_MISSING_ERROR = 'handle_param_name_missing_error'
ON_TYPE_MATCH_ERROR = 'handle_type_match_error'
ON_VALID_TYPE_ERROR = 'handle_valid_type_error'
ON_VALUE_MATCH_ERROR = 'handle_value_match_error'

BEFORE_REQUEST_COMPONENT_CREATE = 'handle_before_request_component_create'
BEFORE_REQUEST_COMPONENT_DELETE = 'handle_before_request_component_delete'