(window.webpackJsonp=window.webpackJsonp||[]).push([[47],{tgWo:function(n,t,o){"use strict";o.r(t),o.d(t,"HomeModuleNgFactory",function(){return h});var e=o("8Y7J");class r{}var l=o("pMnS");class u{constructor(){}ngOnInit(){}}var c=e.yb({encapsulation:2,styles:[],data:{}});function a(n){return e.bc(0,[],null,null)}function s(n){return e.bc(0,[(n()(),e.Ab(0,0,null,null,1,"home",[],null,null,null,a,c)),e.zb(1,114688,null,0,u,[],null,null)],function(n,t){n(t,1,0)},null)}var i=e.wb("home",u,s,{},{},[]),b=o("SVse"),p=o("iInd"),f=o("pfe1"),v=o("AytR");let w=(()=>{class n{constructor(n){this.router=n}resolve(n){v.a.isAgent?this.router.navigate(["/","agent","requests"]):window.location.href=v.a.homepageUrl||"/login"}}return n.\u0275prov=e.cc({factory:function(){return new n(e.dc(p.p))},token:n,providedIn:"root"}),n})();class d{}var h=e.xb(r,[],function(n){return e.Lb([e.Mb(512,e.j,e.bb,[[8,[l.a,i]],[3,e.j],e.z]),e.Mb(4608,b.q,b.p,[e.w]),e.Mb(1073742336,b.c,b.c,[]),e.Mb(1073742336,p.t,p.t,[[2,p.z],[2,p.p]]),e.Mb(1073742336,d,d,[]),e.Mb(1073742336,r,r,[]),e.Mb(1024,p.n,function(){return[[{path:"",component:u,canActivate:[f.a],resolve:{home:w}}]]},[])])})}}]);