APP_EGRESS_NETWORK = '${APP_NETWORK}.egress'
APP_EGRESS_NETWORK_NAME = 'app.egress'
APP_EGRESS_NETWORK_TEMPLATE = '{network}.egress'
APP_INGRESS_NETWORK = '${APP_NETWORK}.ingress'
APP_INGRESS_NETWORK_NAME = 'app.ingress'
APP_INGRESS_NETWORK_TEMPLATE = '{network}.ingress'
DOCKER_COMPOSE_BASE = '.docker-compose.base.yml'
DOCKER_COMPOSE_BASE_TEMPLATE = '.docker-compose.base.template.yml'
DOCKER_COMPOSE_CUSTOM = 'docker-compose.yml'
DOCKER_COMPOSE_NETWORKS = '.docker-compose.networks.yml'
DOCKERFILE_CONTEXT = '.Dockerfile.context'
DOCKERFILE_SERVICE = 'Dockerfile.source'

# TODO: add scaffold container name templates here

# Example:
# COMPOSE_TEMPLATE = 'docker-compose.{workflow}.yml'

