!function(){function l(l,n){for(var u=0;u<n.length;u++){var e=n[u];e.enumerable=e.enumerable||!1,e.configurable=!0,"value"in e&&(e.writable=!0),Object.defineProperty(l,e.key,e)}}function n(l,n){if(!(l instanceof n))throw new TypeError("Cannot call a class as a function")}(window.webpackJsonp=window.webpackJsonp||[]).push([[23],{oyfN:function(u,e,t){"use strict";t.r(e),t.d(e,"Error404ModuleNgFactory",function(){return F});var o=t("8Y7J"),i=function l(){n(this,l)},a=t("pMnS"),r=t("VDRc"),c=t("/q54"),b=t("l+Q0"),f=t("cUpR"),s=t("sF+I"),h=t.n(s),d=function(){function u(l){n(this,u),this.configService=l,this.icSearch=h.a,this.configService.setConfig("vex-layout-ikaros")}var e,t,o;return e=u,(t=[{key:"ngOnInit",value:function(){}},{key:"ngOnDestroy",value:function(){this.configService.setConfig("vex-layout-zeus")}}])&&l(e.prototype,t),o&&l(e,o),u}(),x=t("lC2v"),p=o.yb({encapsulation:0,styles:[[""]],data:{}});function g(l){return o.bc(0,[(l()(),o.Ab(0,0,null,null,13,"div",[["class","w-full p-gutter text-center"]],null,null,null,null,null)),(l()(),o.Ab(1,0,null,null,0,"img",[["class","w-full max-w-lg mx-auto mb-6"],["src","assets/img/illustrations/data_center.svg"]],null,null,null,null,null)),(l()(),o.Ab(2,0,null,null,1,"h1",[["class","display-3 font-medium m-0"]],null,null,null,null,null)),(l()(),o.Yb(-1,null,["404"])),(l()(),o.Ab(4,0,null,null,1,"h2",[["class","headline m-0"]],null,null,null,null,null)),(l()(),o.Yb(-1,null,["Page not found."])),(l()(),o.Ab(6,0,null,null,7,"div",[["class","bg-card rounded-full overflow-hidden relative ltr:pl-5 rtl:pr-5 h-12 max-w-md w-full shadow-8 mx-auto mt-6"],["fxLayout","row"],["fxLayoutAlign","start center"]],null,null,null,null,null)),o.zb(7,671744,null,0,r.d,[o.l,c.i,r.k,c.f],{fxLayout:[0,"fxLayout"]},null),o.zb(8,671744,null,0,r.c,[o.l,c.i,r.i,c.f],{fxLayoutAlign:[0,"fxLayoutAlign"]},null),(l()(),o.Ab(9,0,null,null,2,"ic-icon",[["class","text-secondary"],["fxFlex","none"],["height","24px"],["width","24px"]],[[2,"ic-inline",null],[4,"font-size",null],[8,"innerHTML",1]],null,null,null,null)),o.zb(10,737280,null,0,r.b,[o.l,c.i,c.e,r.h,c.f],{fxFlex:[0,"fxFlex"]},null),o.zb(11,606208,null,0,b.a,[f.b],{icon:[0,"icon"],width:[1,"width"],height:[2,"height"]},null),(l()(),o.Ab(12,0,null,null,1,"input",[["class","border-0 h-12 outline-none ltr:pl-4 rtl:pr-4 placeholder:text-secondary bg-card"],["fxFlex","auto"],["placeholder","Search for other pages ..."],["type","text"]],null,null,null,null,null)),o.zb(13,737280,null,0,r.b,[o.l,c.i,c.e,r.h,c.f],{fxFlex:[0,"fxFlex"]},null)],function(l,n){var u=n.component;l(n,7,0,"row"),l(n,8,0,"start center"),l(n,10,0,"none"),l(n,11,0,u.icSearch,"24px","24px"),l(n,13,0,"auto")},function(l,n){l(n,9,0,o.Ob(n,11).inline,o.Ob(n,11).size,o.Ob(n,11).iconHTML)})}var w=o.wb("vex-error404",d,function(l){return o.bc(0,[(l()(),o.Ab(0,0,null,null,1,"vex-error404",[],null,null,null,g,p)),o.zb(1,245760,null,0,d,[x.a],null,null)],function(l,n){l(n,1,0)},null)},{},{},[]),y=t("SVse"),v=t("iInd"),M={containerEnabled:!0,toolbarShadowEnabled:!0},m=function l(){n(this,l)},z=t("9gLZ"),A=t("ura0"),L=t("Nhcz"),S=t("u9T3"),F=o.xb(i,[],function(l){return o.Lb([o.Mb(512,o.j,o.bb,[[8,[a.a,w]],[3,o.j],o.z]),o.Mb(4608,y.q,y.p,[o.w]),o.Mb(5120,o.b,function(l,n){return[c.j(l,n)]},[y.d,o.D]),o.Mb(1073742336,y.c,y.c,[]),o.Mb(1073742336,v.t,v.t,[[2,v.z],[2,v.p]]),o.Mb(1073742336,m,m,[]),o.Mb(1073742336,c.c,c.c,[]),o.Mb(1073742336,z.a,z.a,[]),o.Mb(1073742336,r.g,r.g,[]),o.Mb(1073742336,A.c,A.c,[]),o.Mb(1073742336,L.a,L.a,[]),o.Mb(1073742336,S.a,S.a,[c.g,o.D]),o.Mb(1073742336,b.b,b.b,[]),o.Mb(1073742336,i,i,[]),o.Mb(1024,v.n,function(){return[[{path:"",component:d,data:M}]]},[])])})}}])}();