!function(){function n(n,t){for(var e=0;e<t.length;e++){var i=t[e];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(n,i.key,i)}}function t(n,t){if(!(n instanceof t))throw new TypeError("Cannot call a class as a function")}(window.webpackJsonp=window.webpackJsonp||[]).push([[43],{"5Rpo":function(e,i,o){"use strict";o.r(i),o.d(i,"InvitationCallbackModuleNgFactory",function(){return g});var r=o("8Y7J"),a=function n(){t(this,n)},u=o("pMnS"),c=function(){function e(n,i,o,r,a){t(this,e),this.organizationsUserResource=n,this.projectsUserResource=i,this.route=o,this.router=r,this.tokenService=a}var i,o,r;return i=e,(o=[{key:"ngOnInit",value:function(){var n,t,e=this,i=this.route.snapshot,o=i.queryParams,r=o.organization_invitation_token,a=o.project_invitation_token;this.tokenService.userSignedIn?(a?(n=this.projectsUserResource,t=a):r&&(n=this.organizationsUserResource,t=r),n&&t&&n.create({invitation_token:t}).subscribe(function(){e.router.navigate(["/"])})):this.router.navigate(["/login"],{queryParams:i.queryParams})}}])&&n(i.prototype,o),r&&n(i,r),e}(),s=o("YCZM"),l=o("XTWy"),b=o("iInd"),f=o("hU4o"),p=r.yb({encapsulation:0,styles:[[""]],data:{}});function h(n){return r.bc(0,[],null,null)}var v=r.wb("stoobly-invitation-callback",c,function(n){return r.bc(0,[(n()(),r.Ab(0,0,null,null,1,"stoobly-invitation-callback",[],null,null,null,h,p)),r.zb(1,114688,null,0,c,[s.a,l.a,b.a,b.p,f.c],null,null)],function(n,t){n(t,1,0)},null)},{},{},[]),y=o("SVse"),k=function n(){t(this,n)},g=r.xb(a,[],function(n){return r.Lb([r.Mb(512,r.j,r.bb,[[8,[u.a,v]],[3,r.j],r.z]),r.Mb(4608,y.q,y.p,[r.w]),r.Mb(1073742336,y.c,y.c,[]),r.Mb(1073742336,b.t,b.t,[[2,b.z],[2,b.p]]),r.Mb(1073742336,k,k,[]),r.Mb(1073742336,a,a,[]),r.Mb(1024,b.n,function(){return[[{path:"",component:c}]]},[])])})}}])}();