!function(){function n(n,t){for(var e=0;e<t.length;e++){var o=t[e];o.enumerable=o.enumerable||!1,o.configurable=!0,"value"in o&&(o.writable=!0),Object.defineProperty(n,o.key,o)}}function t(t,e,o){return e&&n(t.prototype,e),o&&n(t,o),t}function e(n,t){if(!(n instanceof t))throw new TypeError("Cannot call a class as a function")}(window.webpackJsonp=window.webpackJsonp||[]).push([[47],{tgWo:function(n,o,u){"use strict";u.r(o),u.d(o,"HomeModuleNgFactory",function(){return g});var r=u("8Y7J"),i=function n(){e(this,n)},c=u("pMnS"),l=function(){function n(){e(this,n)}return t(n,[{key:"ngOnInit",value:function(){}}]),n}(),a=r.yb({encapsulation:2,styles:[],data:{}});function f(n){return r.bc(0,[],null,null)}var b,s=r.wb("home",l,function(n){return r.bc(0,[(n()(),r.Ab(0,0,null,null,1,"home",[],null,null,null,f,a)),r.zb(1,114688,null,0,l,[],null,null)],function(n,t){n(t,1,0)},null)},{},{},[]),p=u("SVse"),h=u("iInd"),w=u("pfe1"),v=u("AytR"),d=((b=function(){function n(t){e(this,n),this.router=t}return t(n,[{key:"resolve",value:function(n){v.a.isAgent?this.router.navigate(["/","agent","requests"]):window.location.href=v.a.homepageUrl||"/login"}}]),n}()).\u0275prov=r.cc({factory:function(){return new b(r.dc(h.p))},token:b,providedIn:"root"}),b),y=function n(){e(this,n)},g=r.xb(i,[],function(n){return r.Lb([r.Mb(512,r.j,r.bb,[[8,[c.a,s]],[3,r.j],r.z]),r.Mb(4608,p.q,p.p,[r.w]),r.Mb(1073742336,p.c,p.c,[]),r.Mb(1073742336,h.t,h.t,[[2,h.z],[2,h.p]]),r.Mb(1073742336,y,y,[]),r.Mb(1073742336,i,i,[]),r.Mb(1024,h.n,function(){return[[{path:"",component:l,canActivate:[w.a],resolve:{home:d}}]]},[])])})}}])}();