(window.webpackJsonp=window.webpackJsonp||[]).push([[23],{oyfN:function(l,n,u){"use strict";u.r(n),u.d(n,"Error404ModuleNgFactory",function(){return S});var t=u("8Y7J");class e{}var a=u("pMnS"),o=u("VDRc"),r=u("/q54"),i=u("l+Q0"),c=u("cUpR"),b=u("sF+I"),s=u.n(b);class f{constructor(l){this.configService=l,this.icSearch=s.a,this.configService.setConfig("vex-layout-ikaros")}ngOnInit(){}ngOnDestroy(){this.configService.setConfig("vex-layout-zeus")}}var d=u("lC2v"),x=t.yb({encapsulation:0,styles:[[""]],data:{}});function h(l){return t.bc(0,[(l()(),t.Ab(0,0,null,null,13,"div",[["class","w-full p-gutter text-center"]],null,null,null,null,null)),(l()(),t.Ab(1,0,null,null,0,"img",[["class","w-full max-w-lg mx-auto mb-6"],["src","assets/img/illustrations/data_center.svg"]],null,null,null,null,null)),(l()(),t.Ab(2,0,null,null,1,"h1",[["class","display-3 font-medium m-0"]],null,null,null,null,null)),(l()(),t.Yb(-1,null,["404"])),(l()(),t.Ab(4,0,null,null,1,"h2",[["class","headline m-0"]],null,null,null,null,null)),(l()(),t.Yb(-1,null,["Page not found."])),(l()(),t.Ab(6,0,null,null,7,"div",[["class","bg-card rounded-full overflow-hidden relative ltr:pl-5 rtl:pr-5 h-12 max-w-md w-full shadow-8 mx-auto mt-6"],["fxLayout","row"],["fxLayoutAlign","start center"]],null,null,null,null,null)),t.zb(7,671744,null,0,o.d,[t.l,r.i,o.k,r.f],{fxLayout:[0,"fxLayout"]},null),t.zb(8,671744,null,0,o.c,[t.l,r.i,o.i,r.f],{fxLayoutAlign:[0,"fxLayoutAlign"]},null),(l()(),t.Ab(9,0,null,null,2,"ic-icon",[["class","text-secondary"],["fxFlex","none"],["height","24px"],["width","24px"]],[[2,"ic-inline",null],[4,"font-size",null],[8,"innerHTML",1]],null,null,null,null)),t.zb(10,737280,null,0,o.b,[t.l,r.i,r.e,o.h,r.f],{fxFlex:[0,"fxFlex"]},null),t.zb(11,606208,null,0,i.a,[c.b],{icon:[0,"icon"],width:[1,"width"],height:[2,"height"]},null),(l()(),t.Ab(12,0,null,null,1,"input",[["class","border-0 h-12 outline-none ltr:pl-4 rtl:pr-4 placeholder:text-secondary bg-card"],["fxFlex","auto"],["placeholder","Search for other pages ..."],["type","text"]],null,null,null,null,null)),t.zb(13,737280,null,0,o.b,[t.l,r.i,r.e,o.h,r.f],{fxFlex:[0,"fxFlex"]},null)],function(l,n){var u=n.component;l(n,7,0,"row"),l(n,8,0,"start center"),l(n,10,0,"none"),l(n,11,0,u.icSearch,"24px","24px"),l(n,13,0,"auto")},function(l,n){l(n,9,0,t.Ob(n,11).inline,t.Ob(n,11).size,t.Ob(n,11).iconHTML)})}function p(l){return t.bc(0,[(l()(),t.Ab(0,0,null,null,1,"vex-error404",[],null,null,null,h,x)),t.zb(1,245760,null,0,f,[d.a],null,null)],function(l,n){l(n,1,0)},null)}var g=t.wb("vex-error404",f,p,{},{},[]),w=u("SVse"),v=u("iInd");const M={containerEnabled:!0,toolbarShadowEnabled:!0};class y{}var m=u("9gLZ"),z=u("ura0"),A=u("Nhcz"),L=u("u9T3"),S=t.xb(e,[],function(l){return t.Lb([t.Mb(512,t.j,t.bb,[[8,[a.a,g]],[3,t.j],t.z]),t.Mb(4608,w.q,w.p,[t.w]),t.Mb(5120,t.b,function(l,n){return[r.j(l,n)]},[w.d,t.D]),t.Mb(1073742336,w.c,w.c,[]),t.Mb(1073742336,v.t,v.t,[[2,v.z],[2,v.p]]),t.Mb(1073742336,y,y,[]),t.Mb(1073742336,r.c,r.c,[]),t.Mb(1073742336,m.a,m.a,[]),t.Mb(1073742336,o.g,o.g,[]),t.Mb(1073742336,z.c,z.c,[]),t.Mb(1073742336,A.a,A.a,[]),t.Mb(1073742336,L.a,L.a,[r.g,t.D]),t.Mb(1073742336,i.b,i.b,[]),t.Mb(1073742336,e,e,[]),t.Mb(1024,v.n,function(){return[[{path:"",component:f,data:M}]]},[])])})}}]);